@@ -206,6 +206,7 @@ static bool addArgumentsAPTGet(std::vector<CommandLine::Args> &Args, char const
       addArg(0, "allow-releaseinfo-change-codename", "Acquire::AllowReleaseInfoChange::Codename", 0);
       addArg(0, "allow-releaseinfo-change-suite", "Acquire::AllowReleaseInfoChange::Suite", 0);
       addArg(0, "allow-releaseinfo-change-defaultpin", "Acquire::AllowReleaseInfoChange::DefaultPin", 0);
+      addArg('e', "error-on", "APT::Update::Error-Mode", CommandLine::HasArg);
    }
    else if (CmdMatches("source"))
    {
@@ -506,8 +507,7 @@ static void BinaryCommandSpecificConfiguration(char const * const Binary, char c
       // we support it anyhow, but allow it on the commandline to take effect
       // even through it isn't documented as a user who doesn't want it wouldn't
       // ask for it
-      _config->Set("Binary::apt-get::APT::Get::AutomaticRemove", false);
-      _config->Set("Binary::apt::APT::Get::AutomaticRemove", false);
+      _config->Set("APT::Get::AutomaticRemove", "");
    }
 }
 #undef CmdMatches
