@@ -10,9 +10,7 @@
 #define CONTENTS_H
 
 #include <apt-pkg/dirstream.h>
-#include <apt-pkg/strutl.h>
 
-#include <set>
 #include <string>
 #include <stddef.h>
 #include <stdio.h>
@@ -22,34 +20,51 @@ class FileFd;
 
 class GenContents
 {
-   /// \brief zero-terminated strings inside a BigBlock with minimal C++ accessor and comparison operator
-   struct StringInBlock
+   struct Node
    {
-      const char *data;
-      const char *c_str() const { return data; }
-      bool operator<(const StringInBlock &other) const { return strcmp(data, other.data) < 0; }
+      // Binary Tree links
+      Node *BTreeLeft;
+      Node *BTreeRight;
+      Node *DirDown;
+      Node *Dups;
+      const char *Path;
+      const char *Package;
+      
+      void *operator new(size_t Amount,GenContents *Owner);
+      void operator delete(void *) {};
+      
+      Node() : BTreeLeft(0), BTreeRight(0), DirDown(0), Dups(0), 
+               Path(0), Package(0) {};
    };
+   friend struct Node;
+   
    struct BigBlock
    {
       void *Block;
       BigBlock *Next;
    };
-
-   std::set<std::pair<StringInBlock, StringInBlock>> Entries;
-
+   
+   Node Root;
+   
    // Big block allocation pools
    BigBlock *BlockList;   
    char *StrPool;
    unsigned long StrLeft;
-
+   Node *NodePool;
+   unsigned long NodeLeft;
+   
+   Node *Grab(Node *Top,const char *Name,const char *Package);
    void WriteSpace(std::string &out, size_t Current, size_t Target);
-
+   void DoPrint(FileFd &Out,Node *Top, char *Buf);
+   
    public:
-   StringInBlock Mystrdup(const char *From);
-   void Add(const char *Dir, StringInBlock Package);
+   
+   char *Mystrdup(const char *From);
+   void Add(const char *Dir,const char *Package);   
    void Print(FileFd &Out);
 
-   GenContents() : BlockList(0), StrPool(0), StrLeft(0) {};
+   GenContents() : BlockList(0), StrPool(0), StrLeft(0), 
+                   NodePool(0), NodeLeft(0) {};
    ~GenContents();
 };
 
